package ca.canadapost.cpcdp.ncshipping;

import ca.canadapost.cpcdp.ncshipping.generated.messages.Messages;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import org.apache.commons.io.IOUtils;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

import java.io.FileOutputStream;

/**
 * Sample code for the GetArtifact Canada Post service.
 * <p>
 * The GetArtifact service is used to retrieve a pdf of the shipping label 
 * created by a prior Create Non-Contract Shipment call. 
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetArtifact Request.  <br>
 *  USAGE:<pre>
 *        GetArtifactClient client = new GetArtifactClient(username, password);
 *        ClientResponse response = client.getArtifact();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetArtifactClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username and password are imported from the following file
    	// CPCWS_NCShipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	    	
    	// REST URL - ers
    	// LINK = "https://ct.soa-gw.canadapost.ca/ers/artifact/" + username + "/10238/0";    	
    	
    	// REST URL - rs
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/artifact/" + username + "/10238/0";
    	
    	
		// Execute getArtifact Request
        GetArtifactClient myClient = new GetArtifactClient(username, password);
        ClientResponse resp = myClient.getArtifact();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());
        
        try {
        	String mediaStr = resp.getType().toString();
        
        	if ( mediaStr.contains("application/pdf") ) {
        		// Writing binary response to file
            	String fileName = "artifact.pdf";
                FileOutputStream fos = new FileOutputStream(fileName);
                IOUtils.copy(resp.getEntityInputStream(), fos);                                
                fos.close();
                System.out.println("Writing response to " + System.getProperty("user.dir") + System.getProperty("file.separator") + fileName);   
            }  else {    	        		
                // Example of using JAXB to parse xml error response
	            JAXBContext jc;
	        	jc = JAXBContext.newInstance(Messages.class);
	            Object entity = jc.createUnmarshaller().unmarshal(respIS);
	
	            Messages messageData = (Messages) entity;
	            for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
	                Messages.Message aMessage = (Messages.Message) iter.next();
	                System.out.println("Error Code: " + aMessage.getCode());
	                System.out.println("Error Msg: " + aMessage.getDescription());
	            }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }
        myClient.close();
    }

    public GetArtifactClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getArtifact() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
